@@ -4,8 +4,8 @@
 
 # Sympa - SYsteme de Multi-Postage Automatique
 #
-# Copyright 2017, 2018 The Sympa Community. See the AUTHORS.md file at the
-# top-level directory of this distribution and at
+# Copyright 2017, 2018, 2019, 2020 The Sympa Community. See the AUTHORS.md
+# file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -26,15 +26,15 @@ package Sympa::Request::Handler::create_automatic_list;
 use strict;
 use warnings;
 use English qw(-no_match_vars);
+use File::Copy qw();
 
 use Sympa;
 use Sympa::Aliases;
 use Conf;
-use Sympa::Constants;
+use Sympa::Config_XML;
 use Sympa::List;
 use Sympa::LockedFile;
 use Sympa::Log;
-use Sympa::Regexps;
 use Sympa::Template;
 
 use base qw(Sympa::Request::Handler);
@@ -51,65 +51,32 @@ sub _twist {
     my $request = shift;
 
     my $family         = $request->{context};
-    my $listname       = lc($request->{listname} || '');
     my $param          = $request->{parameters};
     my $abort_on_error = $request->{abort_on_error};
-    my $robot_id       = $family->{'robot'};
+    my $robot_id       = $family->{'domain'};
+
+    my $path;
 
     die 'bug in logic. Ask developer' unless ref $family eq 'Sympa::Family';
-    $family->{'state'} = 'no_check';
 
-    # Check listname.
-    my $listname_re = Sympa::Regexps::listname();
-    unless (defined $listname
-        and $listname =~ /^$listname_re$/i
-        and length $listname <= Sympa::Constants::LIST_LEN()) {
-        $log->syslog('err', 'Incorrect listname %s', $listname);
-        $self->add_stash($request, 'user', 'incorrect_listname',
-            {bad_listname => $listname});
-        return undef;
-    }
+    if ($param->{file}) {
+        $path = $param->{file};
 
-    my $regx = Conf::get_robot_conf($robot_id, 'list_check_regexp');
-    if ($regx) {
-        if ($listname =~ /^(\S+)-($regx)$/) {
+        # Get list data
+        $param = Sympa::Config_XML->new($path)->as_hashref;
+        unless ($param) {
             $log->syslog('err',
-                'Incorrect listname %s matches one of service aliases',
-                $listname);
-            $self->add_stash($request, 'user', 'listname_matches_aliases',
-                {new_listname => $listname});
+                "Error in representation data with these xml data");
+            $self->add_stash($request, 'user', 'XXX');
             return undef;
         }
     }
 
-    if (   $listname eq Conf::get_robot_conf($robot_id, 'email')
-        or $listname eq Conf::get_robot_conf($robot_id, 'listmaster_email')) {
-        $log->syslog('err',
-            'Incorrect listname %s matches one of service aliases',
-            $listname);
-        $self->add_stash($request, 'user', 'listname_matches_aliases',
-            {new_listname => $listname});
-        return undef;
-    }
-
-    ## Check listname on SMTP server
-    my $aliases =
-        Sympa::Aliases->new(Conf::get_robot_conf($robot_id, 'alias_manager'));
-    my $res = $aliases->check($listname, $robot_id) if $aliases;
-    unless (defined $res) {
-        $log->syslog('err', 'Can\'t check list %.128s on %s',
-            $listname, $robot_id);
-        $self->add_stash($request, 'intern');
-        return undef;
-    }
-
-    ## Check this listname doesn't exist already.
-    if ($res or Sympa::List->new($listname, $robot_id, {'just_try' => 1})) {
-        $log->syslog('err',
-            'Could not create list %s: list on %s already exist',
-            $listname, $robot_id);
-        $self->add_stash($request, 'user', 'list_already_exists',
-            {new_listname => $listname});
+    my $listname = lc $param->{listname};
+    # Check new listname.
+    my @stash = Sympa::Aliases::check_new_listname($listname, $robot_id);
+    if (@stash) {
+        $self->add_stash($request, @stash);
         return undef;
     }
 
@@ -124,7 +91,6 @@ sub _twist {
     my $family_config =
         Conf::get_robot_conf($robot_id, 'automatic_list_families');
     $param->{'family_config'} = $family_config->{$family->{'name'}};
-    $param->{'listname'}      = $listname;
 
     my $config = '';
     my $template =
@@ -214,13 +180,23 @@ sub _twist {
 
     ## Create associated files if a template was given.
     my @files_to_parse;
-    foreach my $file (split ',',
+    foreach my $file (split /\s*,\s*/,
         Conf::get_robot_conf($robot_id, 'parsed_family_files')) {
-        $file =~ s{\s}{}g;
+        # Compat. <= 6.2.38: message.* were moved to message_*.
+        $file =~ s/\Amessage[.](header|footer)\b/message_$1/;
+
         push @files_to_parse, $file;
     }
     for my $file (@files_to_parse) {
-        my $template_file = Sympa::search_fullpath($family, $file . ".tt2");
+        # Compat. <= 6.2.38: message.* were obsoleted by message_*.
+        my $file_obs;
+        if ($file =~ /\Amessage_(header|footer)\b(.*)\z/) {
+            $file_obs = "message.$1$2";
+        }
+        my $template_file = Sympa::search_fullpath($family, $file . '.tt2');
+        $template_file = Sympa::search_fullpath($family, $file_obs . '.tt2')
+            if not $template_file and $file_obs;
+
         if (defined $template_file) {
             my $file_content;
             my $template =
@@ -251,8 +227,12 @@ sub _twist {
 
     ## Create list object
     my $list;
-    unless ($list =
-        Sympa::List->new($listname, $robot_id, {skip_sync_admin => 1})) {
+    unless (
+        $list = Sympa::List->new(
+            $listname, $robot_id,
+            {skip_sync_admin => 1, no_check_family => 1}
+        )
+    ) {
         $log->syslog('err', 'Unable to create list %s', $listname);
         $self->add_stash($request, 'intern');
         return undef;
@@ -263,9 +243,10 @@ sub _twist {
     $list->restore_users('owner');
     $list->restore_users('editor');
 
-    if ($listname ne $request->{listname}) {
-        $self->add_stash($request, 'notice', 'listname_lowercased');
-    }
+    #FIXME
+    #if ($listname ne $request->{listname}) {
+    #    $self->add_stash($request, 'notice', 'listname_lowercased');
+    #}
 
     ## Create shared if required.
     #if (defined $list->{'admin'}{'shared_doc'}) {
@@ -311,10 +292,8 @@ sub _twist {
     $list->save_config(Sympa::get_address($family, 'listmaster'));
     $list->{'family'} = $family;
 
-    ## check param_constraint.conf
-    $family->{'state'} = 'normal';
+    # Check param_constraint.conf
     my $error = $family->check_param_constraint($list);
-    $family->{'state'} = 'no_check';
 
     unless (defined $error) {
         $list->set_status_error_config('no_check_rules_family',
@@ -329,15 +308,23 @@ sub _twist {
             {errors => $error});
     }
 
+    # Copy files in the list directory : xml file
+    if ($path and $path ne $list->{'dir'} . '/instance.xml') {
+        unless (File::Copy::copy($path, $list->{'dir'} . '/instance.xml')) {
+            $list->set_status_error_config('error_copy_file',
+                $family->{'name'});
+            $self->add_stash($request, 'intern');
+            $log->syslog('err',
+                'Impossible to copy the XML file in the list directory');
+        }
+    }
+
     ## Synchronize list members if required
     if ($list->has_include_data_sources()) {
         $log->syslog('notice', "Synchronizing list members...");
         $list->sync_include();
     }
 
-    ## END
-    $family->{'state'} = 'normal';
-
     return 1;
 }
 
