@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2017 The Sympa Community. See the AUTHORS.md file at the top-level
-# directory of this distribution and at
+# Copyright 2017, 2019, 2020 The Sympa Community. See the AUTHORS.md
+# file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,6 @@ use English qw(-no_match_vars);
 use File::Copy qw();
 
 use Sympa;
-use Sympa::Alarm;
 use Conf;
 use Sympa::Family;
 use Sympa::List;
@@ -41,6 +40,7 @@ use Sympa::Log;
 use Sympa::Mailer;
 use Sympa::Spindle::ProcessRequest;
 use Sympa::Spool::Incoming;
+use Sympa::Spool::Listmaster;
 use Sympa::Tools::Data;
 
 use base qw(Sympa::Spindle);
@@ -55,7 +55,7 @@ sub _init {
 
     if ($state == 1) {
         # Process grouped notifications.
-        Sympa::Alarm->instance->flush;
+        Sympa::Spool::Listmaster->instance->flush;
     }
 
     1;
@@ -72,7 +72,7 @@ sub _on_success {
                 $self->{distaff}->{directory} . '/' . $handle->basename,
                 $self->{keepcopy} . '/' . $handle->basename
             )
-            ) {
+        ) {
             $log->syslog(
                 'notice',
                 'Could not rename %s/%s to %s/%s: %m',
@@ -245,14 +245,13 @@ sub _twist {
         }
 
         my $spindle_req = Sympa::Spindle::ProcessRequest->new(
-            context      => $dyn_family,
-            action       => 'create_automatic_list',
-            listname     => $listname,
-            parameters   => {},
-            sender       => $sender,
-            smime_signed => $message->{'smime_signed'},
-            md5_check    => $message->{'md5_check'},
-            dkim_pass    => $message->{'dkim_pass'},
+            context          => $dyn_family,
+            action           => 'create_automatic_list',
+            parameters       => {listname => $listname},
+            sender           => $sender,
+            smime_signed     => $message->{'smime_signed'},
+            md5_check        => $message->{'md5_check'},
+            dkim_pass        => $message->{'dkim_pass'},
             scenario_context => {
                 sender             => $sender,
                 message            => $message,
@@ -263,13 +262,20 @@ sub _twist {
         unless ($spindle_req and $spindle_req->spin) {
             $log->syslog('err', 'Cannot create dynamic list %s', $listname);
             return undef;
-        } elsif (not($spindle_req->success
-            and $list = Sympa::List->new($listname, $dyn_family->{robot}))) {
+        } elsif (
+            not($spindle_req->success
+                and $list = Sympa::List->new(
+                    $listname,
+                    $dyn_family->{'domain'},
+                    {just_try => 1}
+                )
+            )
+        ) {
             $log->syslog('err',
                 'Unable to create list %s. Message %s ignored',
                 $listname, $message);
             Sympa::send_notify_to_listmaster(
-                $dyn_family->{'robot'},
+                $dyn_family->{'domain'},
                 'automatic_list_creation_failed',
                 {   'listname' => $listname,
                     'family'   => $dyn_list_family,
@@ -279,10 +285,10 @@ sub _twist {
             );
             Sympa::send_dsn($robot, $message, {}, '5.3.5');
             $log->db_log(
-                'robot'        => $dyn_family->{'robot'},
+                'robot'        => $dyn_family->{'domain'},
                 'list'         => $listname,
                 'action'       => 'process_message',
-                'parameters'   => $msg_id . "," . $dyn_family->{'robot'},
+                'parameters'   => $msg_id . "," . $dyn_family->{'domain'},
                 'target_email' => '',
                 'msg_id'       => $msg_id,
                 'status'       => 'error',
@@ -321,10 +327,10 @@ sub _twist {
             # purge the unwanted empty automatic list
             if ($Conf::Conf{'automatic_list_removal'} =~ /if_empty/i) {
                 Sympa::Spindle::ProcessRequest->new(
-                    context      => $robot,
-                    action       => 'close_list',
-                    current_list => $list,
-                    mode         => 'purge',
+                    context          => $robot,
+                    action           => 'close_list',
+                    current_list     => $list,
+                    mode             => 'purge',
                     scenario_context => {skip => 1},
                 )->spin;
             }
@@ -350,10 +356,10 @@ sub _twist {
             # purge the unwanted empty automatic list
             if ($Conf::Conf{'automatic_list_removal'} =~ /if_empty/i) {
                 Sympa::Spindle::ProcessRequest->new(
-                    context      => $robot,
-                    action       => 'close_list',
-                    current_list => $list,
-                    mode         => 'purge',
+                    context          => $robot,
+                    action           => 'close_list',
+                    current_list     => $list,
+                    mode             => 'purge',
                     scenario_context => {skip => 1},
                 )->spin;
             }
