@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2017, 2018 The Sympa Community. See the AUTHORS.md file at the
-# top-level directory of this distribution and at
+# Copyright 2017, 2018, 2019, 2020 The Sympa Community. See the AUTHORS.md
+# file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -29,10 +29,11 @@ package Sympa::Upgrade;
 
 use strict;
 use warnings;
+use Cwd qw();
 use Encode qw();
 use English qw(-no_match_vars);
+use File::Copy qw();
 use MIME::Base64 qw();
-use POSIX qw();
 use Time::Local qw();
 
 use Sympa;
@@ -46,7 +47,6 @@ use Sympa::LockedFile;
 use Sympa::Log;
 use Sympa::Message;
 use Sympa::Request;
-use Sympa::Scenario;
 use Sympa::Spool;
 use Sympa::Spool::Archive;
 use Sympa::Spool::Auth;
@@ -127,26 +127,55 @@ sub upgrade {
         return undef;
     }
 
-    #XXX# Always update config.bin files while upgrading
-    #XXXConf::delete_binaries();
+    # As of 6.2.33b.1, owners/moderators are no longer stored in config file.
+    # - Write out initial permanent owners/editors in <role>.dump files.
+    # - And, if list is not closed, import owners/moderators from those files
+    #   into database.
+    if (lower_version($previous_version, '6.2.33b.1')) {
+        $log->syslog('notice',
+            'Restoring users of ALL lists...it may take a while...');
 
-    ## Always update config.bin files while upgrading
-    ## This is especially useful for character encoding reasons
+        my $all_lists = Sympa::List::get_lists('*', skip_sync_admin => 1);
+        foreach my $list (@{$all_lists || []}) {
+            next unless $list;
+            my $dir = $list->{'dir'};
+
+            my $fh;
+            next unless open $fh, '<', $dir . '/config';
+            my $config = do { local $RS; <$fh> };
+            close $fh;
+
+            $config =~ s/(\A|\n)[\t ]+(?=\n)/$1/g;    # normalize empty lines
+            open my $ifh, '<', \$config;              # open "in memory" file
+            my @config = do { local $RS = ''; <$ifh> };
+            close $ifh;
+            foreach my $role (qw(owner editor)) {
+                my $file = $dir . '/' . $role . '.dump';
+                if (!-e $file and open my $ofh, '>', $file) {
+                    my $admins = join '', grep {/\A\s*$role\b/} @config;
+                    print $ofh $admins;
+                    close $ofh;
+                }
+
+                next
+                    if $list->{'admin'}{'status'} eq 'closed'
+                    or $list->{'admin'}{'status'} eq 'family_closed';
+                $list->restore_users($role);
+            }
+        }
+    }
+
+    # Always update config.bin files while upgrading.
+    # This is especially useful for character encoding reasons.
     $log->syslog('notice',
         'Rebuilding config.bin files for ALL lists...it may take a while...');
-    my $all_lists = Sympa::List::get_lists('*', 'reload_config' => 1);
-
-    ## Empty the admin_table entries and recreate them
-    $log->syslog('notice', 'Rebuilding the admin_table...');
-
-    if ($all_lists and @$all_lists) {
-        foreach my $list (@$all_lists) {
-            $list->sync_include_admin;
-        }
-    } else {
-        # Prevent empty admin table (GH #71).
-        $log->syslog('notice',
-            'Skipping rebuild, no list config files found');
+    my $all_lists =
+        Sympa::List::get_lists('*', reload_config => 1, skip_sync_admin => 1);
+    # Recreate admin_table entries.
+    $log->syslog('notice',
+        'Rebuilding the admin_table...it may take a while...');
+    foreach my $list (@{$all_lists || []}) {    # See GH #71
+        $list->sync_include_admin;
     }
 
     ## Migration to tt2
@@ -439,11 +468,11 @@ sub upgrade {
                 my $rows;
                 $sth = $sdm->do_query(
                     q{UPDATE subscriber_table
-		      SET subscribed_subscriber = 1
-		      WHERE (included_subscriber IS NULL OR
-			     included_subscriber <> 1) AND
-			    (subscribed_subscriber IS NULL OR
-			     subscribed_subscriber <> 1)}
+                      SET subscribed_subscriber = 1
+                      WHERE (included_subscriber IS NULL OR
+                             included_subscriber <> 1) AND
+                            (subscribed_subscriber IS NULL OR
+                             subscribed_subscriber <> 1)}
                 );
                 unless ($sth) {
                     $log->syslog('err', 'Unable to execute SQL statement');
@@ -836,36 +865,58 @@ sub upgrade {
         $log->syslog('notice', 'Fixing robot column of exclusion table');
         my $sth;
         my $sdm = Sympa::DatabaseManager->instance;
-        unless ($sdm
-            and $sth = $sdm->do_query("SELECT * FROM exclusion_table")) {
+        unless (
+            $sdm and $sth = $sdm->do_query(
+                q{SELECT *
+                  FROM exclusion_table
+                  WHERE robot_exclusion IS NULL OR robot_exclusion = ''}
+            )
+        ) {
             $log->syslog('err',
                 'Unable to gather information from the exclusions table');
         }
+        my $rows = $sth->fetchall_arrayref(
+            {list_exclusion => 1, user_exclusion => 1});
+        $sth->finish;
+
         my @robots = Sympa::List::get_robots();
-        while (my $data = $sth->fetchrow_hashref) {
-            next
-                if (defined $data->{'robot_exclusion'}
-                && $data->{'robot_exclusion'} ne '');
-            ## Guessing right robot for each exclusion.
-            my $valid_robot = '';
+        foreach my $data (@{$rows || []}) {
+            # Guessing right robot for each exclusion.
             my @valid_robot_candidates;
             foreach my $robot (@robots) {
-                if (my $list =
-                    Sympa::List->new($data->{'list_exclusion'}, $robot)) {
-                    if ($list->is_list_member($data->{'user_exclusion'})) {
-                        push @valid_robot_candidates, $robot;
-                    }
-                }
+                next
+                    unless Sympa::List->new($data->{'list_exclusion'},
+                    $robot, {just_try => 1});
+                push @valid_robot_candidates, $robot;
             }
-            if ($#valid_robot_candidates == 0) {
-                $valid_robot = $valid_robot_candidates[0];
-                my $sth;
+            unless (@valid_robot_candidates) {
                 unless (
-                    $sth = $sdm->do_query(
-                        "UPDATE exclusion_table SET robot_exclusion = %s WHERE list_exclusion=%s AND user_exclusion=%s",
-                        $sdm->quote($valid_robot),
-                        $sdm->quote($data->{'list_exclusion'}),
-                        $sdm->quote($data->{'user_exclusion'})
+                    $sdm->do_prepared_query(
+                        q{DELETE FROM exclusion_table
+                          WHERE (robot_exclusion IS NULL OR
+                                 robot_exclusion = '') AND
+                                list_exclusion = ? AND user_exclusion = ?},
+                        $data->{list_exclusion}, $data->{user_exclusion}
+                    )
+                ) {
+                    $log->syslog(
+                        'err',
+                        'Unable to delete entry (%s, %s) in exclusions table',
+                        $data->{'list_exclusion'},
+                        $data->{'user_exclusion'}
+                    );
+                }
+            } elsif (1 == scalar @valid_robot_candidates) {
+                my $valid_robot = $valid_robot_candidates[0];
+                unless (
+                    $sdm->do_prepared_query(
+                        q{UPDATE exclusion_table
+                          SET robot_exclusion = ?
+                          WHERE (robot_exclusion IS NULL OR
+                                 robot_exclusion = '') AND
+                                list_exclusion = ? AND user_exclusion = ?},
+                        $valid_robot,
+                        $data->{'list_exclusion'}, $data->{'user_exclusion'}
                     )
                 ) {
                     $log->syslog(
@@ -882,7 +933,7 @@ sub upgrade {
                     'Exclusion robot could not be guessed for user "%s" in list "%s". Either this user is no longer subscribed to the list or the list appears in more than one robot (or the query to the database failed). Here is the list of robots in which this list name appears: "%s"',
                     $data->{'user_exclusion'},
                     $data->{'list_exclusion'},
-                    @valid_robot_candidates
+                    join(', ', @valid_robot_candidates)
                 );
             }
         }
@@ -905,7 +956,7 @@ sub upgrade {
         my $fh;
         my %migrated = ();
         my @newconf  = ();
-        my $date;
+        my ($date, $human_date);
 
         ## Some sympa.conf parameters were overridden by wwsympa.conf.
         ## Others prefer sympa.conf.
@@ -949,8 +1000,9 @@ sub upgrade {
 
         ## Set language of new file content
         $language->push_lang($Conf::Conf{'lang'});
-        $date =
-            $language->gettext_strftime("%d.%b.%Y-%H.%M.%S", localtime time);
+        $date       = time;
+        $human_date = $language->gettext_strftime('%d %b %Y at %H:%M:%S',
+            localtime $date);
 
         if (-r $wwsympa_conf) {
             ## load only sympa.conf
@@ -1048,7 +1100,7 @@ sub upgrade {
                 . ('#' x 76) . "\n" . '#### '
                 . $language->gettext("Migration from wwsympa.conf") . "\n"
                 . '#### '
-                . $date . "\n"
+                . $human_date . "\n"
                 . ('#' x 76) . "\n\n";
 
             foreach my $type (qw(duplicate add obsolete unknown)) {
@@ -1238,19 +1290,9 @@ sub upgrade {
                 unless ref $list->{'admin'}{'archive'} eq 'HASH';
 
             if ($list->{'admin'}{'archive'}{'access'}) {
-                my $scenario = Sympa::Scenario->new(
-                    'function'  => 'archive_mail_access',
-                    'robot'     => $list->{domain},
-                    'name'      => $list->{'admin'}{'archive'}{'access'},
-                    'directory' => $list->{dir}
-                );
-                $list->{'admin'}{'archive'}{'mail_access'} = {
-                    'file_path' => $scenario->{'file_path'},
-                    'name'      => $scenario->{'name'}
-                };
-
+                $list->{'admin'}{'archive'}{'mail_access'} =
+                    {'name' => $list->{'admin'}{'archive'}{'access'}};
             }
-
             delete $list->{'admin'}{'archive'}{'access'};
 
             if (ref $list->{'admin'}{'web_archive'} eq 'HASH'
@@ -1278,8 +1320,11 @@ sub upgrade {
         $log->syslog('notice',
             'Setting web interface colors to new defaults.');
         fix_colors(Sympa::Constants::CONFIG);
-        $log->syslog('info', 'Saving main web_tt2 directory');
-        save_web_tt2("$Conf::Conf{'etc'}/web_tt2");
+
+        if (-d "$Conf::Conf{'etc'}/web_tt2") {
+            $log->syslog('info', 'Saving main web_tt2 directory');
+            save_web_tt2("$Conf::Conf{'etc'}/web_tt2");
+        }
         my @robots = Sympa::List::get_robots();
         foreach my $robot (@robots) {
             if (-f "$Conf::Conf{'etc'}/$robot/robot.conf") {
@@ -1676,7 +1721,7 @@ sub upgrade {
                 my $req_string = do { local $RS; <$lock_fh> };
 
                 # First line of the file contains the user email address +
-                # his/her name.
+                # their name.
                 my ($email, $gecos);
                 if ($req_string =~ s/\A((\S+|\".*\")\@\S+)(?:\t(.*))?\n+//) {
                     ($email, $gecos) = ($1, $2);
@@ -1792,24 +1837,6 @@ sub upgrade {
                 rename $dir . '/subscribers.closed.dump',
                     $dir . '/member.dump';
             }
-
-            my $fh;
-            next unless open $fh, '<', $dir . '/config';
-            my $config = do { local $RS; <$fh> };
-            close $fh;
-            # Write out initial permanent owners/editors in <role>.dump files.
-            $config =~ s/(\A|\n)[\t ]+(?=\n)/$1/g;    # normalize empty lines
-            open my $ifh, '<', \$config;              # open "in memory" file
-            my @config = do { local $RS = ''; <$ifh> };
-            close $ifh;
-            foreach my $role (qw(owner editor)) {
-                my $file = $dir . '/' . $role . '.dump';
-                if (!-e $file and open my $ofh, '>', $file) {
-                    my $admins = join '', grep {/\A\s*$role\b/} @config;
-                    print $ofh $admins;
-                    close $ofh;
-                }
-            }
         }
     }
 
@@ -1861,6 +1888,193 @@ sub upgrade {
         }
     }
 
+    # Task files are moved.
+    if (lower_version($previous_version, '6.2.37b.2')) {
+        my $sitedir = $Conf::Conf{'etc'};
+        my @robotdirs =
+            map { $Conf::Conf{'etc'} . '/' . $_ } Sympa::List::get_robots();
+        my @listdirs =
+            map { $_->{'dir'} } @{Sympa::List::get_lists('*') || []};
+
+        my $model_dir = $sitedir . '/global_task_models';
+        if (-e $model_dir) {
+            my $task_dir = $sitedir . '/tasks';
+            unless (Sympa::Tools::File::copy_dir($model_dir, $task_dir)) {
+                $log->syslog('err', 'Unable to copy %s to %s',
+                    $model_dir, $task_dir);
+            }
+        }
+        foreach my $dir (($sitedir, @robotdirs, @listdirs)) {
+            my $model_dir = $dir . '/list_task_models';
+            if (-e $model_dir) {
+                my $task_dir = $dir . '/tasks';
+                unless (Sympa::Tools::File::copy_dir($model_dir, $task_dir)) {
+                    $log->syslog('err', 'Unable to copy %s to %s',
+                        $model_dir, $task_dir);
+                }
+            }
+        }
+    }
+
+    # Default list scenario names will be specified in robot.conf/sympa.conf
+    # instead of creating symbolic links named "*.default".
+    if (lower_version($previous_version, '6.2.41b.1')) {
+        my @scenarios = qw(visibility
+            send info subscribe add unsubscribe del invite remind review
+            d_read d_edit
+            archive_web_access archive_mail_access
+            tracking);
+
+        my @dirs = ($Conf::Conf{'etc'});
+        push @dirs,
+            map { $Conf::Conf{'etc'} . '/' . $_ } Sympa::List::get_robots();
+        foreach my $dir (@dirs) {
+            my %scenario_names;
+
+            next unless $dir eq $Conf::Conf{'etc'} or -e $dir . '/robot.conf';
+            next unless -d $dir . '/scenari';
+
+            foreach my $scenario (@scenarios) {
+                my $path = $dir . '/scenari/' . $scenario . '.default';
+                next unless -e $path;
+
+                my $name = '';
+                if (-l $path) {
+                    $path = Cwd::abs_path($path);
+                    next unless $path and -e $path;
+
+                    if ($path =~ m{/$scenario\.([-\w]+)\z}) {
+                        $name = $1;
+                    }
+                }
+                $scenario_names{$scenario} = $name;
+            }
+            next unless %scenario_names;
+
+            my $ofh;
+            if ($dir eq $Conf::Conf{'etc'}) {
+                open $ofh, '>>', Conf::get_sympa_conf() or next;
+            } else {
+                open $ofh, '>>', $dir . '/robot.conf' or next;
+            }
+            print $ofh "\n\n";
+            printf $ofh
+                "# Following parameters were added during upgrade to %s\n\n",
+                $new_version;
+            foreach my $scenario (@scenarios) {
+                unless (exists $scenario_names{$scenario}) {
+                    next;
+                } elsif ($scenario_names{$scenario}) {
+                    printf $ofh "%s %s\n", $scenario,
+                        $scenario_names{$scenario};
+                } else {
+                    printf $ofh
+                        "#%s (Assign default manually, or distribution default will be used.)\n",
+                        $scenario;
+                }
+            }
+            close $ofh;
+        }
+    }
+
+    if (lower_version($previous_version, '6.2.41b.1')) {
+        # The header/footer files were renamed.
+        # Site-level files were moved.
+        my %file_map = (
+            'message.header'        => 'message_header',
+            'message.footer'        => 'message_footer',
+            'message.global_footer' => 'message_global_footer'
+        );
+
+        my $all_lists = Sympa::List::get_lists('*');
+        foreach my $list (@{$all_lists || []}) {
+            my $dir = $list->{'dir'};
+            foreach my $file (keys %file_map) {
+                next if $file eq 'message.global_footer';
+
+                my $new_file = $file_map{$file};
+
+                if (-e $dir . '/' . $file and !-e $dir . '/' . $new_file) {
+                    File::Copy::copy($dir . '/' . $file,
+                        $dir . '/' . $new_file);
+                }
+                if (-e $dir . '/' . $file . '.mime'
+                    and !-e $dir . '/' . $new_file . '.mime') {
+                    File::Copy::copy(
+                        $dir . '/' . $file . '.mime',
+                        $dir . '/' . $new_file . '.mime'
+                    );
+                }
+            }
+        }
+        my $dir = $Conf::Conf{'etc'};
+        foreach my $file (keys %file_map) {
+            my $new_file = $file_map{$file};
+
+            if (-e $dir . '/mail_tt2/' . $file) {
+                File::Copy::copy($dir . '/mail_tt2/' . $file,
+                    $dir . '/' . $new_file);
+            }
+            if (-e $dir . '/mail_tt2/' . $file . '.mime') {
+                File::Copy::copy(
+                    $dir . '/mail_tt2/' . $file . '.mime',
+                    $dir . '/' . $new_file . '.mime'
+                );
+            }
+        }
+    }
+
+    # Clean style sheets with earlier timestamp so that they will be recreated
+    # with recent timestamp.
+    if (lower_version($previous_version, '6.2.41b.1')
+        and not lower_version($previous_version, '6.2.26')) {
+        if ($Conf::Conf{'css_path'} and -d $Conf::Conf{'css_path'}) {
+            my @robot_ids = Sympa::List::get_robots();
+            foreach my $robot_id (@robot_ids) {
+                my $dir = $Conf::Conf{'css_path'} . '/' . $robot_id;
+                next unless -e $dir . '/style.css';
+                unlink $dir . '/style.css';
+            }
+        }
+    }
+
+    # Previously shared repository could not be disabled.
+    if (lower_version($previous_version, '6.2.41b.2')) {
+        my $human_date = $language->gettext_strftime('%d %b %Y at %H:%M:%S',
+            localtime time);
+
+        open my $ofh, '>>', Conf::get_sympa_conf();
+        printf $ofh "\n\n# Upgrade from %s to %s\n# %s\nshared_feature on\n",
+            $previous_version, $new_version, $human_date;
+        close $ofh;
+    }
+
+    # included_* and include_sources_* were deprecated and inclusion_*
+    # was introduced in subscriber_table and admin_table.
+    if (lower_version($previous_version, '6.2.45b.1')) {
+        my $sdm = Sympa::DatabaseManager->instance;
+
+        $log->syslog('notice', 'Upgrading subscriber_table and admin_table.');
+        foreach my $role (qw(member owner editor)) {
+            my ($t, $r) =
+                  ($role eq 'member')
+                ? ('subscriber', '')
+                : ('admin',
+                sprintf ' AND role_admin = %s', $sdm->quote($role));
+            unless (
+                $sdm and $sdm->do_prepared_query(
+                    qq{UPDATE ${t}_table
+                       SET inclusion_$t = update_epoch_$t
+                       WHERE included_$t = 1 AND inclusion_$t IS NULL$r}
+                )
+            ) {
+                $log->syslog('err',
+                    'Can\'t update inclusion_%s field for %s in %s_table',
+                    $t, $role, $t);
+            }
+        }
+    }
+
     return 1;
 }
 
@@ -1881,6 +2095,15 @@ sub to_utf8 {
 				   message_report.tt2 | moderate.tt2 |  modindex.tt2 | send_auth.tt2 }x;
     my $total;
 
+    # Get an obsoleted parameter filesystem_encoding.
+    my $filesystem_encoding;
+    open my $fh, '<', Conf::get_sympa_conf();
+    while (my $line = <$fh>) {
+        $filesystem_encoding = $1
+            if $line =~ /\A\s*(?:web_recode_to|filesystem_encoding)\s+(\S+)/i;
+    }
+    close $fh;
+
     foreach my $pair (@{$files}) {
         my ($file, $lang) = @$pair;
         unless (open(TEMPLATE, $file)) {
@@ -1891,12 +2114,11 @@ sub to_utf8 {
         my $text     = '';
         my $modified = 0;
 
-        ## If filesystem_encoding is set, files are supposed to be encoded
-        ## according to it
+        # If an obsoleted parameter filesystem_encoding was set, files are
+        # supposed to be encoded according to it.
         my $charset;
-        if (defined $Conf::Ignored_Conf{'filesystem_encoding'}
-            and $Conf::Ignored_Conf{'filesystem_encoding'} ne 'utf-8') {
-            $charset = $Conf::Ignored_Conf{'filesystem_encoding'};
+        if ($filesystem_encoding) {
+            $charset = $filesystem_encoding;
         } else {
             $language->push_lang($lang);
             $charset = Conf::lang2charset($language->get_lang);
@@ -1950,8 +2172,8 @@ sub to_utf8 {
 
         next unless $modified;
 
-        my $date = POSIX::strftime("%Y.%m.%d-%H.%M.%S", localtime(time));
-        unless (rename $file, $file . '@' . $date) {
+        my $date = time;
+        unless (rename $file, $file . '.' . $date) {
             $log->syslog('err', "Cannot rename old template %s", $file);
             next;
         }
@@ -1974,7 +2196,7 @@ sub to_utf8 {
             next;
         }
         $log->syslog('notice', 'Modified file %s; original file kept as %s',
-            $file, $file . '@' . $date);
+            $file, $file . '.' . $date);
 
         $total++;
     }
@@ -2060,8 +2282,7 @@ sub fix_colors {
         $new_conf .= "$line\n";
     }
     # Save previous config file
-    my $date =
-        $language->gettext_strftime("%d.%b.%Y-%H.%M.%S", localtime time);
+    my $date = time;
     unless (rename($file, "$file.upgrade$date")) {
         $log->syslog(
             'err',
@@ -2104,8 +2325,7 @@ sub save_web_tt2 {
         );
         return 0;
     }
-    my $date =
-        $language->gettext_strftime("%d.%b.%Y-%H.%M.%S", localtime time);
+    my $date = time;
     unless (rename($dir, "$dir.upgrade$date")) {
         $log->syslog(
             'err',
