@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2017 The Sympa Community. See the AUTHORS.md file at the top-level
-# directory of this distribution and at
+# Copyright 2017, 2018, 2019 The Sympa Community. See the AUTHORS.md file
+# at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -104,9 +104,7 @@ sub lists {
         my $listname = $list->{'name'};
 
         my $result_item = {};
-        my $result      = Sympa::Scenario::request_action(
-            $list,
-            'visibility',
+        my $result = Sympa::Scenario->new($list, 'visibility')->authz(
             'md5',
             {   'sender'                  => $sender,
                 'remote_application_name' => $ENV{'remote_application_name'}
@@ -193,8 +191,8 @@ sub login {
     }
 
     ## Create Sympa::WWW::Session object
-    my $session = Sympa::WWW::Session->new($robot,
-        {'cookie' => Sympa::WWW::Session::encrypt_session_id($ENV{'SESSION_ID'})});
+    my $session =
+        Sympa::WWW::Session->new($robot, {cookie => $ENV{SESSION_ID}});
     $ENV{'USER_EMAIL'} = $email;
     $session->{'email'} = $email;
     $session->store();
@@ -204,7 +202,7 @@ sub login {
 
     ## Also return the cookie value
     return SOAP::Data->name('result')->type('string')
-        ->value(Sympa::WWW::Session::encrypt_session_id($ENV{'SESSION_ID'}));
+        ->value($ENV{SESSION_ID});
 }
 
 sub casLogin {
@@ -272,8 +270,11 @@ sub casLogin {
     }
 
     ## Now fetch email attribute from LDAP
-    unless ($email =
-        Sympa::WWW::Auth::get_email_by_net_id($robot, $cas_id, {'uid' => $user})) {
+    unless (
+        $email = Sympa::WWW::Auth::get_email_by_net_id(
+            $robot, $cas_id, {'uid' => $user}
+        )
+    ) {
         $log->syslog('err',
             'Could not get email address from LDAP for user %s', $user);
         die SOAP::Fault->faultcode('Server')
@@ -282,8 +283,8 @@ sub casLogin {
     }
 
     ## Create Sympa::WWW::Session object
-    my $session = Sympa::WWW::Session->new($robot,
-        {'cookie' => Sympa::WWW::Session::encrypt_session_id($ENV{'SESSION_ID'})});
+    my $session =
+        Sympa::WWW::Session->new($robot, {cookie => $ENV{SESSION_ID}});
     $ENV{'USER_EMAIL'} = $email;
     $session->{'email'} = $email;
     $session->store();
@@ -293,7 +294,7 @@ sub casLogin {
 
     ## Also return the cookie value
     return SOAP::Data->name('result')->type('string')
-        ->value(Sympa::WWW::Session::encrypt_session_id($ENV{'SESSION_ID'}));
+        ->value($ENV{SESSION_ID});
 }
 
 ## Used to call a service as an authenticated user without using HTTP cookies
@@ -319,7 +320,7 @@ sub authenticateAndRun {
     ## Provided email is not trusted, we fetch the user email from the
     ## session_table instead
     my $session =
-        Sympa::WWW::Session->new($ENV{'SYMPA_ROBOT'}, {'cookie' => $cookie});
+        Sympa::WWW::Session->new($ENV{'SYMPA_ROBOT'}, {cookie => $cookie});
     if (defined $session) {
         $email      = $session->{'email'};
         $session_id = $session->{'id_session'};
@@ -388,8 +389,8 @@ sub authenticateRemoteAppAndRun {
             ->faultdetail('Use : <appname> <apppassword> <vars> <service>');
     }
     my ($proxy_vars, $set_vars) =
-        Sympa::WWW::Auth::remote_app_check_password($appname, $apppassword, $robot,
-        $service);
+        Sympa::WWW::Auth::remote_app_check_password($appname, $apppassword,
+        $robot, $service);
 
     unless (defined $proxy_vars) {
         $log->syslog('notice', 'Authentication failed');
@@ -498,8 +499,8 @@ sub info {
             ->faultdetail("List $listname unknown");
     }
 
-    my $result = Sympa::Scenario::request_action(
-        $list, 'info', 'md5',
+    my $result = Sympa::Scenario->new($list, 'info')->authz(
+        'md5',
         {   'sender'                  => $sender,
             'remote_application_name' => $ENV{'remote_application_name'}
         }
@@ -560,9 +561,11 @@ sub info {
 }
 
 sub createList {
-    $log->syslog('info', 
+    $log->syslog(
+        'info',
         '(%s, listname=%s, subject=%s, template=%s, description=%s, topics=%s)',
-        @_);
+        @_
+    );
     my $class       = shift;
     my $listname    = shift;
     my $subject     = shift;
@@ -612,17 +615,18 @@ sub createList {
         if Sympa::User::is_global_user($sender);
 
     my $spindle = Sympa::Spindle::ProcessRequest->new(
-        context      => $robot,
-        action       => 'create_list',
-        listname     => $listname,
+        context    => $robot,
+        action     => 'create_list',
         parameters => {
-            owner => [{
-                email => $sender,
-                gecos => ($user ? $user->{gecos} : undef),
-            }],
+            listname => $listname,
+            owner    => [
+                {   email => $sender,
+                    gecos => ($user ? $user->{gecos} : undef),
+                }
+            ],
             subject        => $subject,
             creation_email => $sender,
-            template       => $list_tpl,
+            type           => $list_tpl,
             topics         => $topics,
             description    => $description,
         },
@@ -706,9 +710,9 @@ sub closeList {
     }
 
     my $spindle = Sympa::Spindle::ProcessRequest->new(
-        context          => $list,
-        action           => 'close_list',
-        current_list     => $list,
+        context      => $list->{'domain'},
+        action       => 'close_list',
+        current_list => $list,
         mode =>
             (($list->{'admin'}{'status'} eq 'pending') ? 'purge' : 'close'),
         sender           => $sender,
@@ -950,8 +954,8 @@ sub review {
     # Part of the authorization code
     $user = Sympa::User::get_global_user($sender);
 
-    my $result = Sympa::Scenario::request_action(
-        $list, 'review', 'md5',
+    my $result = Sympa::Scenario->new($list, 'review')->authz(
+        'md5',
         {   'sender'                  => $sender,
             'remote_application_name' => $ENV{'remote_application_name'}
         }
@@ -1278,9 +1282,7 @@ sub which {
 
         my $result_item;
 
-        my $result = Sympa::Scenario::request_action(
-            $list,
-            'visibility',
+        my $result = Sympa::Scenario->new($list, 'visibility')->authz(
             'md5',
             {   'sender'                  => $sender,
                 'remote_application_name' => $ENV{'remote_application_name'}
